// Tiny Whoop camera mount for LST-S2
// Material: nylon

// Parameters

// Render quality / Качество обработки
$fn=128; // [32:32:128]
// Diameter of mounting holes / Диаметр крепежных отверстий
mountholes = 1.6; // [0.8:0.05:3]
// Thickness of base mount / Толщина основания
mountheight = 0.7; // [0.2:0.05:2]
// Distance to horizontal bar / Расстояние до горизонтальной планки
mountlenght = 13; // [1:0.5:20]
// Thickness of lens mount / Толщина основания крепления линзы
lensheight = 0.7; // [0.2:0.05:1.5]
// Distance to lens mount / Расстояние до основания крепления линзы
lensroot = 2; // [2:0.1:8]
// Distance to lens ring / Расстояние до хомута крепления линзы
lenslenght = 5.7; // [1.5:0.1:20]
// Lens mount angle / Угол наклона крепления линзы
lensangle = 71; // [0:0.5:120]
// Camera width / Ширина камеры
camwidth = 15.2; // [0:0.05:28]
// Camera shift / Смещение камеры
camshift = 0; // [-14:0.05:14]
// Lenght of hook / Длина крюка
hooklenght = 13.4; // [0:0.05:30]
// Hook angle / Угол наклона крюка
hookangle = 70; // [0:0.5:120]
// Print longest corners and hook for cutting / Печать длинных уголков и крюка
printlong = false; // [true,false]
// Print lens mount support / Печать поддержки для крепления линзы
printsupport = false; // [true,false]
// Show LST-S2 camera / Показывать камеру LST-S2
showcamera = true; // [true,false]
// Camera angle / Угол наклона камеры
showcameraangle = 20; // [0:0.5:90]

//-------------------------------------------------------------------------
// Do not edit following lines
//-------------------------------------------------------------------------

include <camera.scad>

if ((mountlenght-2)-abs(camshift)-camwidth/2 < 2.5 ) {
    echo ("<b><font color='red'>Cam width or shift must be shorten</b>");
    assert(false);
}

if (mountholes<0.8 || mountholes>3) {
    echo ("<b><font color='red'>Mount holes diameter must be between 0.8-3.0 mm</b>");
    assert(false);
}

if (lensangle<0 || lensangle>120) {
    echo ("<b><font color='red'>Lens angle must be between 0-120 degree</b>");
    assert(false);
}

if (hookangle<0 || hookangle>120) {
    echo ("<b><font color='red'>Hook angle must be between 0-120 degree</b>");
    assert(false);
}

if (mountlenght-lenslenght-lensroot<4.2 && lensangle==0) {
    echo ("<b><font color='red'>Lens lenght or root must be shorten</b>");
    assert(false);
}

if (mountlenght<1 || mountlenght>20) {
    echo ("<b><font color='red'>Mount lenght must be between 1-20 mm</b>");
    assert(false);
}

if (lenslenght<0) {
    echo ("<b><font color='red'>Lens lenght must be longer</b>");
    assert(false);
}

if (lensheight<0.2 || lensheight>1.5) {
    echo ("<b><font color='red'>Lens mount height must be between 0.2-1.5</b>");
    assert(false);
}

if (mountlenght-lensroot<3) {
    echo ("<b><font color='red'>Lens root must be shorten</b>");
    assert(false);
}

module camera_mount() {
    difference() {
        union() {
            mount();
            if (hooklenght>5) hook();
            corners();
            lens();
        }
        // kerfs
        if (lensangle==0) translate([-lensroot,0,mountheight+lensheight+0.15]) rotate([90,0,0]) cylinder(r=0.3,h=3.82,center=true);
        if (hookangle==0) translate([0.09-mountlenght-mountheight/2,0,mountheight+0.15]) rotate([90,0,0]) cylinder(r=0.3,h=2.62,center=true);
    }
    color([1,0,0]) if (printsupport) {
        support();
        mirror([0,1,0]) support();
    }
}

module support() {
    if (mountlenght-lenslenght-lensroot<7) {
        translate([-mountlenght-3.3,-3.3,mountheight/2]) cube([6,3,mountheight],center=true);
    }
    translate([0,1.5,0]) linear_extrude(height=mountheight,center=false) polygon(points=[[-3.5,0],[-mountlenght+2.9,0],[-mountlenght+2.9,4]]);
}

module mount() {
    mount_size=18.385;
    difference() {
        union() {
            // left mount
            translate([-mount_size,-mount_size,0]) cylinder(r=2.1,h=mountheight);
            // right mount
            translate([-mount_size,mount_size,0]) cylinder(r=2.1,h=mountheight);
            // front mount
            translate([0,0,0]) cylinder(r=2.1,h=mountheight+lensheight+0.1);
            // right bar
            translate([-9.5,-8.8,mountheight/2]) rotate([0,0,43]) cube([26,2.3,mountheight],center=true); 
            translate([-9.7,-9.625,mountheight]) rotate([90,0,43]) linear_extrude(height=1.2,center=true) polygon(points=[[0,0],[12,0],[12,lensheight]]);
            // left bar
            translate([-9.5,8.8,mountheight/2]) rotate([0,0,-43]) cube([26,2.3,mountheight],center=true); 
            translate([-9.7,9.625,mountheight]) rotate([90,0,-43]) linear_extrude(height=1.2,center=true) polygon(points=[[0,0],[12,0],[12,lensheight]]);
            // lens root
            translate([-lensroot/2,0,mountheight+(lensheight+0.1)/2]) cube([lensroot,3.8,lensheight+0.1],center=true);
            // corner axis for lens mount
            translate([-lensroot,0,mountheight+(lensheight/2)+0.1]) rotate([90,0,0]) cylinder(r=lensheight/2,h=3.8,center=true);
            // center bar
            translate([-(mountheight/4)-mountlenght/2,0,mountheight/2]) cube([mountlenght+(mountheight/2),2.6,mountheight],center=true);
            // horisontal bar
            translate([-mountlenght+1.3,0,mountheight/2]) cube([2.6,(mountlenght-2)*2,mountheight],center=true);
           // corner axis for hook
            translate([-mountlenght-mountheight/2,0,mountheight/2]) rotate([90,0,0]) cylinder(r=mountheight/2,h=2.6,center=true);
        }
        // mount holes
        translate([-mount_size,-mount_size,-1]) cylinder(r=mountholes/2,h=mountheight+2);
        translate([-mount_size,mount_size,-1]) cylinder(r=mountholes/2,h=mountheight+2);
        translate([0,0,-0.5]) cylinder(r=mountholes/2,h=mountheight+lensheight+2);
    }
 }

module corners() {
    cornerlong=(printlong)?10:2.6;
    translate([-mountlenght+1.2,0,mountheight])  rotate([90,0,0]) linear_extrude(height=2.6,center=true) polygon(points=[[0,0],[-0.5,2],[-0.3,2],[2.5,0]]);
    translate([2.6-mountlenght-cornerlong/2,camshift+camwidth/2,0]) rotate([90,0,90]) linear_extrude(height=cornerlong,center=true) polygon(points=[[0,0],[0,2+mountheight],[0.2,2+mountheight],[2,mountheight],[2,0]]);
    translate([2.6-mountlenght-cornerlong/2,camshift-camwidth/2,0]) rotate([90,0,-90]) linear_extrude(height=cornerlong,center=true) polygon(points=[[0,0],[0,2+mountheight],[0.2,2+mountheight],[2,mountheight],[2,0]]);
}

module lens() {
    translate([-lensroot,0,mountheight+0.1+lensheight/2]) rotate([0,lensangle,0]) union() {
        translate([-lenslenght/2,0,0]) cube([lenslenght,3.8,lensheight],center=true);
        translate([-lenslenght-3.1,0,0.75-lensheight/2]) linear_extrude(height=1.5,center=true) import("lensfix.svg",convexity=10);
    }
}

module hook() {
    hooklong=(printlong && hookangle==0)?16:2.6;
    translate([-mountlenght-mountheight/2,0,mountheight/2]) rotate([0,hookangle,0]) union() {
        // beam
        translate([-1-hooklenght/2,0,0]) cube([hooklenght+2,2.6,mountheight],center=true);
        // hook
        translate([-0.5-hooklenght,0,1.5+mountheight/2]) rotate([90,0,-180]) linear_extrude(height=hooklong,center=true) import("hook.svg",convexity=10);
        translate([-0.9-hooklenght,0,0]) cube([2.2,hooklong,mountheight],center=true);
    }
}

// Show camera
if (showcamera) translate([-mountlenght,camshift,mountheight]) rotate([0,-abs(showcameraangle),0]) camera();
// Show camera mount
color([1,1,0]) camera_mount();
